# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import, unicode_literals

import logbook


QUIET_LOG_LEVEL = logbook.NOTSET


class BuildAction(object):
    CMAKE = "cmake"
    RECMAKE = "recmake"
    CLEAN = "clean"
    BUILD = "build"
    REBUILD = "rebuild"

    DEFAULT = BUILD
    LIST = [CMAKE, RECMAKE, CLEAN, BUILD, REBUILD]


class BuildType(object):
    DEBUG = "Debug"
    RELEASE = "Release"

    DEFAULT = DEBUG
    LIST = [DEBUG, RELEASE]



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import, unicode_literals

import argparse
from textwrap import dedent

import logbook

from .__version__ import __version__
from ._common import QUIET_LOG_LEVEL, BuildAction, BuildType


DEFAULT_CMAKE_OPTIONS_FILE = "cmake_options.json"


def parse_option():
    description = "A CLI tool for CMake and compiler wrapper."
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=description,
        epilog=dedent(
            """\
            Issue tracker: https://github.com/thombashi/cmakew/issues
            """
        ),
    )
    parser.add_argument("-V", "--version", action="version", version="%(prog)s " + __version__)

    parser.add_argument(
        "source_dir", metavar="SOURCE_DIR_PATH", help="""relative path to the source directory."""
    )

    group = parser.add_argument_group("Directory Options")
    group.add_argument(
        "--build-dir",
        default="build",
        help="""
        relative path to the build output directory
        (defaults to '%(default)s').
        """,
    )

    group = parser.add_argument_group("Build Options")
    group.add_argument(
        "--action",
        choices=BuildAction.LIST,
        default=BuildAction.DEFAULT,
        help="""
        cmake: execute CMake and exit.
        clean: delete existing build directory and exit.
        recmake: delete existing CMakeCache and execute CMake after that.
        build: execute MSBuild to Visual Studio solution files that created by cmake.
        rebuild: delete existing build directory and execute CMake and
        MSBuild after that.
        defaults to '%(default)s'.
        """,
    )

    group = parser.add_argument_group("CMake Options")
    group.add_argument(
        "--cmake-options",
        default=DEFAULT_CMAKE_OPTIONS_FILE,
        help="""
        path to the CMake options file. use "{key :value, ...}"
        to set specific parameters. defaults to %(default)s.
        """,
    )
    group.add_argument(
        "--build-type",
        choices=BuildType.LIST,
        default=BuildType.DEFAULT,
        help="defaults to %(default)s.",
    )
    group.add_argument(
        "--generator",
        help="""
        generator that passes to cmake.
        the default value decided by execution platform:
        (a) if executed at Windows and Visual Studio is installed in C: or D:
        drive, cmakew pass 'Visual Studio NN'  as a generator to cmake.
        (b) "Unix Makefiles" otherwise
        """,
    )

    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "--debug",
        dest="log_level",
        action="store_const",
        const=logbook.DEBUG,
        default=logbook.INFO,
        help="for debug print.",
    )
    group.add_argument(
        "--quiet",
        dest="log_level",
        action="store_const",
        const=QUIET_LOG_LEVEL,
        default=logbook.INFO,
        help="suppress execution log messages.",
    )

    return parser.parse_args()



#!/usr/bin/env python
# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import, unicode_literals

import abc
import os.path
import re
import shutil

import six
import subprocrunner
import typepy

from ._logger import logger
from ._visual_studio import find_vs_solution_file_list, vsinfo


@six.add_metaclass(abc.ABCMeta)
class CompilerInterface(object):
    @abc.abstractmethod
    def build(self):
        pass

    @abc.abstractmethod
    def clean(self):
        pass


class AbstractCompiler(CompilerInterface):

    _VALIDATE_ERROR_MSG_TEMPLATE = "build path must not a root directory path: actual={}"

    @property
    def build_dir_path(self):
        return self.__build_dir_path

    def __init__(self, build_dir_path):
        self._validate_build_dir_path(build_dir_path)
        self.__build_dir_path = os.path.abspath(os.path.expanduser(build_dir_path))

    def _validate_build_dir_path(self, dir_path):
        if typepy.is_null_string(dir_path):
            raise ValueError("build path must not an empty directory path")

        if dir_path == "/":
            raise ValueError(self._VALIDATE_ERROR_MSG_TEMPLATE.format(dir_path))

    def _run_build(self, build_command):
        runner = subprocrunner.SubprocessRunner(build_command)
        runner.run()

        if typepy.is_not_null_string(runner.stdout):
            logger.info(runner.stdout)

        if typepy.is_null_string(runner.stderr):
            return

        if runner.returncode == 0:
            logger.info(runner.stderr)
        else:
            logger.error(runner.stderr)

    def clean(self):
        self._validate_build_dir_path(self.build_dir_path)

        logger.debug('delete "{:s}" directory'.format(self.build_dir_path))
        shutil.rmtree(self.build_dir_path, ignore_errors=True)

        return 0


class WindowsCompiler(AbstractCompiler):
    def _validate_build_dir_path(self, dir_path):
        super(WindowsCompiler, self)._validate_build_dir_path(dir_path)

        if re.search(r"^[a-zA-Z]:\\?$", dir_path) is not None:
            raise ValueError(self._VALIDATE_ERROR_MSG_TEMPLATE.format(dir_path))

    def build(self):
        for solution_file in find_vs_solution_file_list(self.build_dir_path):
            self._run_build('"{:s}" {:s}'.format(vsinfo.msbuild_path, solution_file))


class LnuxCompiler(AbstractCompiler):
    def build(self):
        import multiprocessing

        try:
            make_command = "make -j{:d}".format(multiprocessing.cpu_count())
        except NotImplementedError:
            make_command = "make"

        self._run_build(make_command)


class CompilerFactory(object):
    @staticmethod
    def create(build_dir_path):
        import platform

        if platform.system() == "Windows":
            return WindowsCompiler(build_dir_path)

        return LnuxCompiler(build_dir_path)



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import

import os

from ._logger import logger


class Cd(object):
    def __init__(self, new_path):
        self.__new_path = os.path.expanduser(new_path)
        self.__saved_path = None

    def __enter__(self):
        self.__saved_path = os.getcwd()
        logger.debug("change directory to {}".format(self.__new_path))
        os.chdir(self.__new_path)

    def __exit__(self, exception_type, exception_value, traceback):
        logger.debug("change directory to {}".format(self.__saved_path))
        os.chdir(self.__saved_path)



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import, unicode_literals

import itertools
import os.path
import platform
import re
from collections import namedtuple


def find_vs_solution_file_list(root_path):
    re_solution = re.compile("[\.]sln$")
    solution_file_path_list = []

    for filename in os.listdir(root_path):
        if re_solution.search(filename) is None:
            continue

        solution_file_path_list.append("/".join([root_path, filename]))

    return solution_file_path_list


class VisualStudioInfo(object):
    __RE_VS_DIR_NAME = re.compile("Microsoft Visual Studio [0-9]+[\.][0-9]+")
    __RE_VS_VERSION = re.compile("[0-9]+[\.][0-9]+")

    VersionInfo = namedtuple("VersionInfo", "major minor")

    @property
    def version_info(self):
        return self.__max_version_info

    @property
    def msbuild_path(self):
        return self.__msbuild_path

    def __init__(self, search_drive_list=None):
        self.__version_info_set = set()
        self.__max_version_info = None
        self.__msbuild_path = None

        if platform.system() != "Windows":
            return

        if not search_drive_list:
            search_drive_list = ["C:"]

        self.__program_files_dir_list = ["Program Files", "Program Files (x86)"]

        self.__detect_version(search_drive_list)
        self.__detect_msbuild(search_drive_list)

    def __detect_version(self, search_drive_list):
        max_vs_version = 0

        for search_drive, program_files_dir in itertools.product(
            search_drive_list, self.__program_files_dir_list
        ):

            try:
                dir_list = os.listdir("{:s}\\{:s}".format(search_drive, program_files_dir))
            except WindowsError:
                continue

            for dir_name in dir_list:
                match = self.__RE_VS_DIR_NAME.search(dir_name)
                if match is None:
                    continue

                version_string = self.__RE_VS_VERSION.search(match.group()).group()
                vs_version = float(version_string)
                version_info = self.VersionInfo(*[int(ver) for ver in version_string.split(".")])
                self.__version_info_set.add(version_info)

                if vs_version > max_vs_version:
                    max_vs_version = vs_version
                    self.__max_version_info = version_info

    def __detect_msbuild(self, search_drive_list):
        for search_drive, program_files_dir in itertools.product(
            search_drive_list, self.__program_files_dir_list
        ):

            try:
                dir_list = os.listdir("{:s}\\{:s}".format(search_drive, program_files_dir))
            except WindowsError:
                continue

            for dir_name in dir_list:
                if dir_name != "MSBuild":
                    continue

                for version_info in reversed(sorted(self.__version_info_set)):
                    msbuild_path = "/".join(
                        [
                            search_drive,
                            program_files_dir,
                            dir_name,
                            "{:d}.{:d}".format(version_info.major, version_info.minor),
                            "Bin",
                            "MSBuild.exe",
                        ]
                    )

                    if os.path.isfile(msbuild_path):
                        self.__msbuild_path = msbuild_path
                        return


vsinfo = VisualStudioInfo()



#!/usr/bin/env python
# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import, unicode_literals

import errno
import json
import os
import platform
import sys

import logbook
import six
import subprocrunner
import typepy
from logbook.more import ColorizedStderrHandler

from ._cd import Cd
from ._common import BuildAction
from ._compiler import CompilerFactory
from ._logger import logger, set_log_level
from ._option import parse_option
from ._visual_studio import vsinfo


ColorizedStderrHandler(
    level=logbook.DEBUG, format_string="[{record.level_name}] {record.channel}: {record.message}"
).push_application()


class CMakeCommandBuilder(object):
    def __init__(self, options):
        self.__options = options

    def get_cmake_commmand(self):
        cmake_command_list = [
            "cmake ../{:s}".format(self.__options.source_dir),
            "-DCMAKE_BUILD_TYPE={:s}".format(self.__options.build_type),
        ]

        for key, value in six.iteritems(self.__read_cmake_options()):
            cmake_command_list.append("-D{}={}".format(key, value))

        generator = self.__get_generator()
        if generator is not None:
            cmake_command_list.append('-G "{:s}"'.format(generator))

        return " ".join(cmake_command_list)

    @staticmethod
    def __get_win_generator():
        return "Visual Studio {:d} {:s}".format(
            vsinfo.version_info.major, "Win64" if platform.architecture()[0] == "64bit" else ""
        )

    def __get_generator(self):
        if self.__options.generator is not None:
            return self.__options.generator

        system = platform.system()

        if system == "Windows":
            return self.__get_win_generator()
        if system == "Linux":
            return "Unix Makefiles"

        logger.debug("unexpected system '{}', considered as Linux".format(system))

        return "Unix Makefiles"

    def __read_cmake_options(self):
        file_path = self.__options.cmake_options

        if typepy.is_null_string(file_path):
            logger.debug("no cmake option file designated.")
            return {}

        if not os.path.isfile(file_path):
            logger.debug("cmake option file not found: path='{}'".format(file_path))
            return {}

        with open(file_path) as f:
            cmake_options = json.loads(f.read())

        return cmake_options


def main():
    options = parse_option()
    set_log_level(options.log_level)

    build_dir = options.build_dir
    try:
        compiler = CompilerFactory.create(build_dir)
    except ValueError as e:
        logger.error("{:s}: {}".format(e.__class__.__name__, e))
        return errno.EINVAL

    if options.action in [BuildAction.CLEAN, BuildAction.REBUILD]:
        result = compiler.clean()

        if options.action == BuildAction.CLEAN:
            return result

    if options.action in [BuildAction.RECMAKE]:
        cmake_cache_path = "/".join([compiler.build_dir_path, "CMakeCache.txt"])
        logger.debug("delete {:s}".format(cmake_cache_path))
        os.remove(cmake_cache_path)

    if not os.path.isdir(build_dir):
        logger.debug("make directory: {:s}".format(build_dir))
        os.makedirs(build_dir)

    if options.action in [
        BuildAction.CMAKE,
        BuildAction.RECMAKE,
        BuildAction.BUILD,
        BuildAction.REBUILD,
    ]:
        command_builder = CMakeCommandBuilder(options)
        runner = subprocrunner.SubprocessRunner(command_builder.get_cmake_commmand())

        with Cd(build_dir):
            runner.run()

        if runner.returncode == 0:
            logging_func_stdout = logger.info
            logging_func_stderr = logger.warn
        else:
            logging_func_stdout = logger.info
            logging_func_stderr = logger.error

        if typepy.is_not_null_string(runner.stdout):
            logging_func_stdout(runner.stdout)

        if typepy.is_not_null_string(runner.stderr):
            logging_func_stderr(runner.stderr)

        if runner.returncode != 0:
            return 1

    if options.action in [BuildAction.BUILD, BuildAction.REBUILD]:
        with Cd(build_dir):
            compiler.build()

    return 0


if __name__ == "__main__":
    sys.exit(main())
